
#line 3 "<stdout>"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 4
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* begin standard C++ headers. */

/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define YY_SC_TO_UI(c) ((YY_CHAR) (c))

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START
/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin  )
#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

extern int yyleng;

extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )
#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = NULL; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = NULL;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart ( FILE *input_file  );
void yy_switch_to_buffer ( YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE yy_create_buffer ( FILE *file, int size  );
void yy_delete_buffer ( YY_BUFFER_STATE b  );
void yy_flush_buffer ( YY_BUFFER_STATE b  );
void yypush_buffer_state ( YY_BUFFER_STATE new_buffer  );
void yypop_buffer_state ( void );

static void yyensure_buffer_stack ( void );
static void yy_load_buffer_state ( void );
static void yy_init_buffer ( YY_BUFFER_STATE b, FILE *file  );
#define YY_FLUSH_BUFFER yy_flush_buffer( YY_CURRENT_BUFFER )

YY_BUFFER_STATE yy_scan_buffer ( char *base, yy_size_t size  );
YY_BUFFER_STATE yy_scan_string ( const char *yy_str  );
YY_BUFFER_STATE yy_scan_bytes ( const char *bytes, int len  );

void *yyalloc ( yy_size_t  );
void *yyrealloc ( void *, yy_size_t  );
void yyfree ( void *  );

#define yy_new_buffer yy_create_buffer
#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}
#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}
#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define yywrap() (/*CONSTCOND*/1)
#define YY_SKIP_YYWRAP
typedef flex_uint8_t YY_CHAR;

FILE *yyin = NULL, *yyout = NULL;

typedef int yy_state_type;

extern int yylineno;
int yylineno = 1;

extern char *yytext;
#ifdef yytext_ptr
#undef yytext_ptr
#endif
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state ( void );
static yy_state_type yy_try_NUL_trans ( yy_state_type current_state  );
static int yy_get_next_buffer ( void );
static void yynoreturn yy_fatal_error ( const char* msg  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;
#define YY_NUM_RULES 112
#define YY_END_OF_BUFFER 113
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static const flex_int16_t yy_accept[587] =
    {   0,
        0,    0,  113,  108,  103,  104,   95,  109,  106,   89,
       96,   93,   94,   99,  110,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,   97,  108,  103,
      104,  106,    0,    0,  107,    0,  106,  106,  105,  101,
       99,    0,  102,  108,  108,  108,  108,   21,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,   39,  108,
      108,  108,  108,   34,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,

      108,    0,    0,    0,  100,   99,  102,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,   33,   65,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,   73,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,    0,    0,  100,   99,  108,  108,
      108,   69,  108,  108,   66,   70,  108,  108,  108,   37,
       38,  108,  108,  108,  108,   42,   44,  108,   53,  108,
      108,   62,   72,  108,   74,  108,   75,  108,  108,  108,

      108,  108,  108,  108,   32,  108,   87,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,   77,   78,  108,   64,  108,    0,  100,   99,  108,
       67,  108,  108,  108,  108,  108,   61,  108,   68,  108,
       98,  108,   40,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,   43,  108,  108,   51,  108,
      108,  108,  108,  108,  108,   90,  108,  108,  108,  108,
      108,  108,  108,   88,  100,   99,  108,  108,  108,  108,
      108,   71,  108,  108,  108,  108,  108,  108,   72,   79,
       80,   81,   82,   83,   84,   85,   86,  108,  108,  108,

      108,   63,  108,  108,  108,    1,  108,  108,  108,  108,
      108,   41,   92,   91,  108,   76,  108,  108,  108,  108,
      100,   99,  108,  108,  108,  108,  108,  108,  108,  108,
       52,  108,   23,   28,  108,  108,  108,   12,  108,  108,
      108,  108,  108,   54,  108,  108,  108,  108,  108,   47,
       64,  100,   99,  108,   11,   69,  108,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,   49,  108,
      108,  108,  108,  108,   50,  108,   48,  108,  100,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,   14,
      108,  108,  108,  108,  108,  108,   46,   13,  108,  108,

      108,  108,  108,  108,  100,  108,  108,  108,  108,  108,
       35,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,  108,  108,   10,  108,  108,  108,  108,  108,  108,
      108,  108,  108,  108,  108,   45,   36,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
      108,   22,  108,  108,  108,  108,   55,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
       58,  108,  108,  108,  108,  108,   56,  108,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,  108,
       59,  108,  108,   17,  108,  108,  108,  108,   19,    7,

      108,  108,  108,  108,  108,  108,  108,   25,  108,  108,
      108,  108,  108,  108,  108,  108,  108,  108,  108,    4,
      108,  108,  108,   24,  108,  108,  108,  108,  108,   15,
      108,  108,  108,  108,    5,    6,  108,  108,  108,  108,
      108,  108,  108,   18,  108,  108,  108,  108,  108,  108,
        3,   27,  108,  108,   20,  108,  108,  108,   29,  108,
       57,  108,   26,  108,   16,   60,  108,  108,  108,  108,
      108,   30,  108,  108,  108,  108,  108,   31,    2,  108,
      108,  108,  108,    9,    8,    0
    } ;

static const YY_CHAR yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    5,    6,    7,    8,    9,    8,   10,
       11,    8,    8,    8,   12,    8,    8,   13,   14,   15,
       16,   17,   18,   19,   20,   21,   21,    8,    8,    8,
        8,    8,    8,    8,   22,   22,   22,   22,   22,   22,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        8,   23,    8,    8,   24,    8,   25,   26,   27,   28,

       29,   30,   31,   32,   33,   34,   35,   36,   37,   38,
       39,   40,   41,   42,   43,   44,   45,   46,   47,   48,
       49,   50,    8,   51,    8,    8,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const YY_CHAR yy_meta[52] =
    {   0,
        1,    2,    3,    1,    2,    1,    1,    1,    1,    1,
        1,    1,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1
    } ;

static const flex_int16_t yy_base[593] =
    {   0,
        0,    0,  801,    0,   50,  802,    0,   49,   55,    0,
        0,    0,    0,   67,   59,  767,   19,   49,   34,   56,
      766,   29,   39,   59,  769,   30,   71,   70,  751,  771,
      750,   76,   74,  755,   65,  764,   82,    0,    0,  111,
      802,  789,   73,   97,  802,  134,  123,  788,  802,  170,
      179,  109,  119,  752,   93,  745,   94,    0,   94,  759,
      761,  743,   98,  756,  738,  753,  740,  752,    0,  741,
      750,  734,  741,    0,  738,  113,  733,  747,  731,  739,
      728,  723,  120,  733,  739,  119,  734,  728,  133,  118,
      720,  725,  735,  722,  718,  720,  730,  731,  715,  714,

      722,  163,    0,  196,  214,  223,  157,  727,  711,  713,
      719,  714,  722,  704,  709,  709,  700,  717,  713,  713,
      699,  707,    0,  700,  711,  711,  696,  710,  694,  694,
      696,  692,  692,  704,    0,  692,  684,  683,  135,  692,
      688,  681,  689,  696,  676,  688,  690,  688,  671,  686,
      669,  667,  700,  667,  668,  140,  669,  672,  671,  664,
      665,  661,  665,  658,    0,  232,  250,  259,  665,  656,
      652,  658,  664,  664,    0,    0,  656,  663,  654,    0,
        0,  661,  649,  648,  644,    0,    0,  661,    0,  645,
      641,    0,  656,  648,    0,  658,    0,  649,  175,  654,

      668,  637,  651,  640,    0,  647,    0,  648,  641,  629,
      630,  638,  641,  641,  626,  625,  628,  620,  622,  624,
      633,  649,    0,  627,  626,  629,  276,  287,  296,  612,
        0,  644,  613,  616,  641,  614,    0,  613,    0,  611,
      606,  604,    0,  618,  618,  609,  305,  613,  600,  598,
      601,  611,  609,  605,  608,    0,  592,  593,    0,  601,
      606,  594,  592,  601,  587,    0,  584,  598,  595,  579,
      582,  584,  584,    0,  313,  322,  593,  590,  586,  587,
      576,    0,  578,  577,  602,  584,  600,  582,    0,    0,
        0,    0,    0,    0,    0,    0,    0,  581,  582,  596,

      567,    0,  578,  577,  560,  591,  573,  557,  571,  553,
      551,    0,    0,    0,  554,    0,  567,  568,  556,  562,
      331,  340,  563,  563,  544,  577,  543,  558,  549,  128,
      573,  554,  571,    0,  543,  552,  538,    0,  550,  549,
      547,  564,  526,  562,  540,  534,  528,  534,  539,    0,
        0,  349,  556,  555,    0,    0,  251,  533,  538,  535,
      518,  523,  145,  533,  255,  522,  521,  529,    0,  515,
      528,  525,  253,  512,    0,  541,    0,  519,  358,  353,
      509,  507,  505,  504,  509,  503,  533,  499,  501,    0,
      509,  511,  501,  501,  492,  494,    0,    0,  508,  493,

      491,  494,  507,  495,    0,  494,  480,  485,  488,  497,
        0,  513,  480,  491,  351,  489,  482,  478,  490,  479,
      488,  483,  472,    0,  475,  484,  474,  469,  481,  476,
      465,  478,  461,  463,  472,    0,    0,  470,   95,  466,
      464,  471,  451,  463,  470,  454,  483,  449,  451,  455,
      460,    0,  461,  462,  446,  447,  474,  440,  448,  455,
      439,  437,  468,  452,  445,  432,  432,  436,  449,  433,
      460,  442,  427,  431,  429,  455,    0,  423,  425,  435,
      434,  426,  428,  257,  416,  416,  418,  445,  418,  413,
        0,  411,  415,    0,  408,  413,  408,  417,    0,    0,

      406,  408,  408,  412,  419,  404,  398,  429,  408,  402,
      407,  405,  392,  423,  404,  406,  403,  389,  399,    0,
      387,  384,  385,    0,  358,  381,  396,  381,  394,    0,
      395,  388,  390,  383,    0,    0,  389,  385,  367,  370,
      357,  361,  359,    0,  359,  354,  361,  358,  354,  362,
        0,    0,  352,  348,    0,  342,  349,  346,    0,  346,
        0,  251,    0,  250,    0,    0,  263,  263,  251,  260,
      254,    0,  244,  244,  224,  159,  164,    0,    0,  178,
      170,  158,  153,    0,    0,  802,  399,  403,  407,  411,
      173,  166

    } ;

static const flex_int16_t yy_def[593] =
    {   0,
      586,    1,  586,  587,  586,  586,  587,  588,  589,  587,
      587,  587,  587,  587,  586,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  586,
      586,  590,  586,  588,  586,  588,  589,  590,  586,  587,
       14,  586,  586,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,

      587,  586,  591,  588,  587,   14,  586,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  592,  104,  587,   14,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,

      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  588,  587,   14,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,   14,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,

      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,   14,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,

      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,

      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,  587,  587,  587,  587,  587,
      587,  587,  587,  587,  587,    0,  586,  586,  586,  586,
      586,  586

    } ;

static const flex_int16_t yy_nxt[854] =
    {   0,
        4,    5,    6,    7,    8,    9,   10,    4,   11,   12,
       13,    4,   14,   14,   14,   14,   14,   14,   14,   14,
       14,    4,   15,   16,   17,    4,   18,   19,   20,   21,
       22,   23,   24,    4,   25,   26,   27,   28,   29,   30,
       31,   32,   33,   34,   35,   36,   37,    4,    4,    4,
       38,   40,   41,   45,   55,   42,   48,   49,   60,   48,
       52,   53,   61,   56,   70,   72,   62,   73,   78,   79,
       71,   46,   43,   57,   52,   53,   58,   48,   50,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   74,   75,
       59,   63,   64,   65,   66,   80,   76,   67,   82,   81,

       88,   45,   90,   68,   89,   91,  100,   97,   83,   98,
       52,   53,   40,   41,   84,   92,   42,   93,   94,   46,
      107,  109,   95,  101,   48,   49,  114,   48,  110,  112,
      119,  139,  115,   43,  586,  102,   44,  113,  460,  131,
      120,   43,  132,  461,  151,   48,  103,  103,  103,  103,
      103,  103,  103,  103,  103,  586,  143,  147,  107,  152,
      144,  153,  198,  140,  102,   44,  148,  361,  216,   44,
      388,  362,  217,  149,  389,  150,  165,  199,  218,   43,
      585,  104,  105,  105,  105,  105,  105,  105,  105,  105,
      105,  106,  106,  106,  106,  106,  106,  106,  106,  106,

       45,  580,  581,  250,  584,  583,  582,  579,  166,  166,
      166,  166,  166,  166,  166,  166,  166,  166,   46,  251,
      166,  166,  166,  166,  166,  166,  167,  167,  167,  167,
      167,  167,  167,  167,  167,  168,  168,  168,  168,  168,
      168,  168,  168,  168,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  578,  577,  227,  227,  227,  227,
      227,  227,  228,  228,  228,  228,  228,  228,  228,  228,
      228,  229,  229,  229,  229,  229,  229,  229,  229,  229,
       45,  381,  391,  400,  504,  576,  575,  392,  574,  573,
      393,  572,  571,  570,  569,  382,  505,  401,   46,  275,

      275,  275,  275,  275,  275,  275,  275,  275,  276,  276,
      276,  276,  276,  276,  276,  276,  276,  290,  291,  292,
      293,  294,  295,  296,  297,  321,  321,  321,  321,  321,
      321,  321,  321,  321,  322,  322,  322,  322,  322,  322,
      322,  322,  322,  352,  352,  352,  352,  352,  352,  352,
      352,  352,  353,  353,  353,  353,  353,  353,  353,  353,
      353,  379,  379,  379,  379,  379,  379,  379,  379,  379,
      405,  405,  405,  405,  405,  405,  405,  405,  405,  406,
      438,  568,  540,  567,  566,  565,  564,  563,  562,  561,
      560,  559,  558,  439,  557,  407,  556,  408,  541,   39,

      555,  554,   39,   44,   44,  553,   44,   47,   47,   47,
       47,   48,   48,   48,   48,  552,  551,  550,  549,  548,
      547,  546,  545,  544,  543,  542,  539,  538,  537,  536,
      535,  534,  533,  532,  531,  530,  529,  528,  527,  526,
      525,  524,  523,  522,  521,  520,  519,  518,  517,  516,
      515,  514,  513,  512,  511,  510,  509,  508,  507,  506,
      503,  502,  501,  500,  499,  498,  497,  496,  495,  494,
      493,  492,  491,  490,  489,  488,  487,  486,  485,  484,
      483,  482,  481,  480,  479,  478,  477,  476,  475,  474,
      473,  472,  471,  470,  469,  468,  467,  466,  465,  464,

      463,  462,  459,  458,  457,  456,  455,  454,  453,  452,
      451,  450,  449,  448,  447,  446,  445,  444,  443,  442,
      441,  440,  437,  436,  435,  434,  433,  432,  431,  430,
      429,  428,  427,  426,  425,  424,  423,  422,  421,  420,
      419,  418,  417,  416,  415,  414,  413,  412,  411,  410,
      409,  404,  403,  402,  399,  398,  397,  396,  395,  394,
      390,  387,  386,  385,  384,  383,  380,   50,  378,  377,
      376,  375,  374,  373,  356,  372,  371,  370,  369,  368,
      367,  366,  365,  364,  363,  360,  359,  358,  357,  356,
      355,  354,  351,  350,  349,  348,  347,  346,  345,  344,

      343,  342,  341,  340,  339,  338,  337,  336,  335,  334,
      333,  332,  331,  330,  329,  328,  327,  326,  325,  324,
      323,  320,  319,  318,  317,  316,  315,  314,  313,  312,
      311,  310,  309,  308,  307,  306,  305,  304,  303,  302,
      301,  300,  299,  298,  289,  288,  287,  286,  285,  284,
      283,  282,  281,  280,  279,  278,  277,  274,  273,  272,
      271,  270,  269,  268,  267,  266,  265,  264,  263,  262,
      261,  260,  259,  258,  257,  256,  255,  239,  254,  253,
      252,  249,  248,  247,  246,  245,  244,  243,  242,  241,
      240,  239,  238,  237,  236,  235,  234,  233,  232,  231,

      230,  226,  225,  224,  223,  222,  221,  220,  219,  215,
      214,  213,  212,  211,  210,  209,  208,  207,  206,  205,
      204,  203,  202,  201,  200,  197,  196,  195,  194,  193,
      192,  191,  190,  189,  188,  187,  186,  185,  184,  183,
      182,  181,  180,  179,  178,  177,  176,  175,  174,  173,
      172,  171,  170,  169,  164,  163,  162,  161,  160,  159,
      158,  157,  156,  155,  154,  146,  145,  142,  141,  138,
      137,  136,  135,  134,  133,  130,  129,  128,  127,  126,
      125,  124,  123,  122,  121,  118,  117,  116,  111,  108,
       49,   49,   99,   96,   87,   86,   85,   77,   69,   54,

      586,    3,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586
    } ;

static const flex_int16_t yy_chk[854] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    5,    5,    8,   17,    5,    9,    9,   19,    9,
       15,   15,   19,   17,   22,   23,   19,   23,   26,   26,
       22,    8,    5,   18,   43,   43,   18,    9,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   24,   24,
       18,   20,   20,   20,   20,   27,   24,   20,   28,   27,

       32,   44,   33,   20,   32,   33,   37,   35,   28,   35,
       52,   52,   40,   40,   28,   33,   40,   33,   33,   44,
       53,   55,   33,   37,   47,   47,   59,   47,   55,   57,
       63,   83,   59,   40,   46,   46,   46,   57,  439,   76,
       63,   53,   76,  439,   90,   47,   46,   46,   46,   46,
       46,   46,   46,   46,   46,   46,   86,   89,  107,   90,
       86,   90,  139,   83,  102,  102,   89,  330,  156,  592,
      363,  330,  156,   89,  363,   89,  591,  139,  156,  107,
      583,   46,   50,   50,   50,   50,   50,   50,   50,   50,
       50,   51,   51,   51,   51,   51,   51,   51,   51,   51,

      104,  577,  577,  199,  582,  581,  580,  576,  104,  104,
      104,  104,  104,  104,  104,  104,  104,  104,  104,  199,
      104,  104,  104,  104,  104,  104,  105,  105,  105,  105,
      105,  105,  105,  105,  105,  106,  106,  106,  106,  106,
      106,  106,  106,  106,  166,  166,  166,  166,  166,  166,
      166,  166,  166,  166,  575,  574,  166,  166,  166,  166,
      166,  166,  167,  167,  167,  167,  167,  167,  167,  167,
      167,  168,  168,  168,  168,  168,  168,  168,  168,  168,
      227,  357,  365,  373,  484,  573,  571,  365,  570,  569,
      365,  568,  567,  564,  562,  357,  484,  373,  227,  228,

      228,  228,  228,  228,  228,  228,  228,  228,  229,  229,
      229,  229,  229,  229,  229,  229,  229,  247,  247,  247,
      247,  247,  247,  247,  247,  275,  275,  275,  275,  275,
      275,  275,  275,  275,  276,  276,  276,  276,  276,  276,
      276,  276,  276,  321,  321,  321,  321,  321,  321,  321,
      321,  321,  322,  322,  322,  322,  322,  322,  322,  322,
      322,  352,  352,  352,  352,  352,  352,  352,  352,  352,
      379,  379,  379,  379,  379,  379,  379,  379,  379,  380,
      415,  560,  525,  558,  557,  556,  554,  553,  550,  549,
      548,  547,  546,  415,  545,  380,  543,  380,  525,  587,

      542,  541,  587,  588,  588,  540,  588,  589,  589,  589,
      589,  590,  590,  590,  590,  539,  538,  537,  534,  533,
      532,  531,  529,  528,  527,  526,  523,  522,  521,  519,
      518,  517,  516,  515,  514,  513,  512,  511,  510,  509,
      508,  507,  506,  505,  504,  503,  502,  501,  498,  497,
      496,  495,  493,  492,  490,  489,  488,  487,  486,  485,
      483,  482,  481,  480,  479,  478,  476,  475,  474,  473,
      472,  471,  470,  469,  468,  467,  466,  465,  464,  463,
      462,  461,  460,  459,  458,  457,  456,  455,  454,  453,
      451,  450,  449,  448,  447,  446,  445,  444,  443,  442,

      441,  440,  438,  435,  434,  433,  432,  431,  430,  429,
      428,  427,  426,  425,  423,  422,  421,  420,  419,  418,
      417,  416,  414,  413,  412,  410,  409,  408,  407,  406,
      404,  403,  402,  401,  400,  399,  396,  395,  394,  393,
      392,  391,  389,  388,  387,  386,  385,  384,  383,  382,
      381,  378,  376,  374,  372,  371,  370,  368,  367,  366,
      364,  362,  361,  360,  359,  358,  354,  353,  349,  348,
      347,  346,  345,  344,  343,  342,  341,  340,  339,  337,
      336,  335,  333,  332,  331,  329,  328,  327,  326,  325,
      324,  323,  320,  319,  318,  317,  315,  311,  310,  309,

      308,  307,  306,  305,  304,  303,  301,  300,  299,  298,
      288,  287,  286,  285,  284,  283,  281,  280,  279,  278,
      277,  273,  272,  271,  270,  269,  268,  267,  265,  264,
      263,  262,  261,  260,  258,  257,  255,  254,  253,  252,
      251,  250,  249,  248,  246,  245,  244,  242,  241,  240,
      238,  236,  235,  234,  233,  232,  230,  226,  225,  224,
      222,  221,  220,  219,  218,  217,  216,  215,  214,  213,
      212,  211,  210,  209,  208,  206,  204,  203,  202,  201,
      200,  198,  196,  194,  193,  191,  190,  188,  185,  184,
      183,  182,  179,  178,  177,  174,  173,  172,  171,  170,

      169,  164,  163,  162,  161,  160,  159,  158,  157,  155,
      154,  153,  152,  151,  150,  149,  148,  147,  146,  145,
      144,  143,  142,  141,  140,  138,  137,  136,  134,  133,
      132,  131,  130,  129,  128,  127,  126,  125,  124,  122,
      121,  120,  119,  118,  117,  116,  115,  114,  113,  112,
      111,  110,  109,  108,  101,  100,   99,   98,   97,   96,
       95,   94,   93,   92,   91,   88,   87,   85,   84,   82,
       81,   80,   79,   78,   77,   75,   73,   72,   71,   70,
       68,   67,   66,   65,   64,   62,   61,   60,   56,   54,
       48,   42,   36,   34,   31,   30,   29,   25,   21,   16,

        3,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586,  586,  586,  586,  586,  586,  586,  586,
      586,  586,  586
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int yy_flex_debug;
int yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "lexer.l"
/*
 * userv is copyright Ian Jackson and other contributors.
 * See README for full authorship information.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with userv; if not, see <http://www.gnu.org/licenses/>.
 */
#line 20 "lexer.l"





#include <syslog.h>
#include <assert.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>
#include <fnmatch.h>
#include <limits.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>

#include "config.h"
#include "common.h"
#include "daemon.h"
#include "lib.h"
#include "both.h"
#include "tokens.h"

#define HYPHEN '-'

typedef int directive_fnt(int dtoken);
static directive_fnt df_reject, df_execute, df_executefrompath;
static directive_fnt df_executefromdirectory, df_executebuiltin;
static directive_fnt df_errorstostderr, df_errorstosyslog, df_errorstofile;
static directive_fnt dfg_fdwant, dfg_setflag, dfg_lookupquotemode;
static directive_fnt df_reset, df_cd, df_userrcfile, df_include;
static directive_fnt df_includelookup, df_includedirectory;
static directive_fnt df_message, df_error, df_quit, df_eof;
static directive_fnt df_if, df_catchquit, df_errorspush;
static directive_fnt dfi_includeuserrcfile, dfi_includeclientconfig;
/* directive functions return:
 *  0 for success having scanned up to and including end of line but not beyond,
 *  or tokv_error or tokv_quit.
 * They expect to parse the whitespace before their parameters (if any).
 */

typedef int parmcondition_fnt(int ctoken, char *const *parmvalues, int *rtrue);
static parmcondition_fnt pcf_glob, pcf_range, pcf_grep;
/* all conditional functions return tokv_error for failure or 0 for success
 *  at parsing and testing, in which case *rtrue is set to 0 or 1.
 *  On success they have scanned up to and including the condition's
 *  terminating newline; the pcf_... functions expect to parse the whitespace
 *  between the parameter name and the condition's arguments.
 * Otherwise they return tokv_error.
 * The parameter-based conditionals take a list of parameter values
 * as obtained from the parameter functions and pa_parameter,
 * and do _not_ free it.
 */

typedef int parameter_fnt(int ptoken, char ***rvalues);
static parameter_fnt pf_service;
static parameter_fnt pf_callinguser, pf_serviceuser;
static parameter_fnt pf_callinggroup, pf_servicegroup;
static parameter_fnt pf_callingusershell, pf_serviceusershell;
/* Parameter functions return tokv_error or 0 for success at parsing
 * and determining the value, in which case *rvalues is made to be
 * a mallocd null-terminated array of pointers to mallocd strings.
 * freeparm can be used to free such an array.
 */

typedef int builtinserviceparse_fnt(char ***rnewargs);
static builtinserviceparse_fnt bispa_none, bispa_parameter;
/* These parse the arguments to a builtin service, including the
 * newline at the end of the line.  *rnewargs will initially be
 * null, indicating that no arguments are to be set; the function
 * may store a mallocd array of mallocd strings in it,
 * containing the arguments it wishes to have set (null-pointer
 * terminated).
 */

static int yylex(void);
/* Returns a token (which may be an eof or error exception) */

static directive_fnt *lr_dir;
static parmcondition_fnt *lr_parmcond;
static builtinserviceparse_fnt *lr_bispa;
static builtinserviceexec_fnt *lr_bisexec;
static parameter_fnt *lr_parameter;
static int lr_loglevel, lr_logfacility, lr_min, lr_max, *lr_flag;
static int lr_flagval, lr_controlend;
static int lr_fdwant_readwrite; /* -1=never, 0=opt, 1=always */

/* Forward declarations of things used in lexer and parser */

struct parser_state {
  int lineno, reportlineno, notedreferer, isinternal;
  const char *filename;
  struct stat filestab;
  YY_BUFFER_STATE ybuf;
  struct parser_state *upstate;
};

static struct parser_state *cstate;

struct error_handling {
  int handling; /* One of the error handling modes tokt_ehandlemode */
  int logfacility, loglevel;
  int filekeep; /* File is in use by higher-level errors-push, leave it open */
  FILE *file;
  char *filename;
};

static struct error_handling eh = { tokv_word_errorstostderr, 0,0,0,0,0 };

static int dequote(char *inplace);
static void countnewlines(void);

#define YY_NO_INPUT

#line 966 "<stdout>"
#line 967 "<stdout>"

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals ( void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy ( void );

int yyget_debug ( void );

void yyset_debug ( int debug_flag  );

YY_EXTRA_TYPE yyget_extra ( void );

void yyset_extra ( YY_EXTRA_TYPE user_defined  );

FILE *yyget_in ( void );

void yyset_in  ( FILE * _in_str  );

FILE *yyget_out ( void );

void yyset_out  ( FILE * _out_str  );

			int yyget_leng ( void );

char *yyget_text ( void );

int yyget_lineno ( void );

void yyset_lineno ( int _line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap ( void );
#else
extern int yywrap ( void );
#endif
#endif

#ifndef YY_NO_UNPUT
    
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy ( char *, const char *, int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen ( const char * );
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput ( void );
#else
static int input ( void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (yy_size_t) max_size, yyin)) == 0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex (void);

#define YY_DECL int yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    
	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer( yyin, YY_BUF_SIZE );
		}

		yy_load_buffer_state(  );
		}

	{
#line 145 "lexer.l"


#line 1185 "<stdout>"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)] ;
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 587 )
					yy_c = yy_meta[yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 802 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 147 "lexer.l"
{ lr_dir= df_reject; return tokv_word_reject; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 148 "lexer.l"
{ lr_dir= df_executefromdirectory; return tokv_word_executefromdirectory; }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 149 "lexer.l"
{ lr_dir= df_executefrompath; return tokv_word_executefrompath; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 150 "lexer.l"
{ lr_dir= df_executebuiltin; return tokv_word_executebuiltin; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 151 "lexer.l"
{ lr_dir= df_errorstostderr; return tokv_word_errorstostderr; }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 152 "lexer.l"
{ lr_dir= df_errorstosyslog; return tokv_word_errorstosyslog; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 153 "lexer.l"
{ lr_dir= df_errorstofile; return tokv_word_errorstofile; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 154 "lexer.l"
{ lr_dir= dfg_lookupquotemode; return tokv_word_includelookupquoteold; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 155 "lexer.l"
{ lr_dir= dfg_lookupquotemode; return tokv_word_includelookupquotenew; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 156 "lexer.l"
{ lr_dir= dfg_fdwant; lr_fdwant_readwrite=1; return tokv_word_requirefd; }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 157 "lexer.l"
{ lr_dir= dfg_fdwant; lr_fdwant_readwrite=0; return tokv_word_allowfd; }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 158 "lexer.l"
{ lr_dir= dfg_fdwant; lr_fdwant_readwrite=0; return tokv_word_nullfd; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 159 "lexer.l"
{ lr_dir= dfg_fdwant; lr_fdwant_readwrite=-1; return tokv_word_rejectfd; }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 160 "lexer.l"
{ lr_dir= dfg_fdwant; lr_fdwant_readwrite=-1; return tokv_word_ignorefd; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 161 "lexer.l"
{ lr_dir= dfg_setflag; lr_flag= &setenvironment; lr_flagval= 1; return tokv_word_setenvironment; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 162 "lexer.l"
{ lr_dir= dfg_setflag; lr_flag= &setenvironment; lr_flagval= 0; return tokv_word_nosetenvironment; }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 163 "lexer.l"
{ lr_dir= dfg_setflag; lr_flag= &suppressargs; lr_flagval= 1; return tokv_word_suppressargs; }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 164 "lexer.l"
{ lr_dir= dfg_setflag; lr_flag= &suppressargs; lr_flagval= 0; return tokv_word_nosuppressargs; }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 165 "lexer.l"
{ lr_dir= dfg_setflag; lr_flag= &disconnecthup; lr_flagval= 1; return tokv_word_disconnecthup; }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 166 "lexer.l"
{ lr_dir= dfg_setflag; lr_flag= &disconnecthup; lr_flagval= 0; return tokv_word_nodisconnecthup; }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 167 "lexer.l"
{ lr_dir= df_cd; return tokv_word_cd; }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 168 "lexer.l"
{ lr_dir= df_userrcfile; return tokv_word_userrcfile; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 169 "lexer.l"
{ lr_dir= df_include; return tokv_word_include; }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 170 "lexer.l"
{ lr_dir= df_include; return tokv_word_includeifexist; }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 171 "lexer.l"
{ lr_dir= df_includelookup; return tokv_word_includelookup; }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 172 "lexer.l"
{ lr_dir= df_includelookup; return tokv_word_includelookupall; }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 173 "lexer.l"
{ lr_dir= df_includedirectory; return tokv_word_includedirectory; }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 174 "lexer.l"
{ lr_dir= df_message; return tokv_word_message; }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 175 "lexer.l"
{ lr_dir= df_include; return tokv_word_includesysconfig; }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 176 "lexer.l"
{ lr_dir= dfi_includeuserrcfile; return tokv_word_includeuserrcfile; }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 177 "lexer.l"
{ lr_dir= dfi_includeclientconfig; return tokv_word_includeclientconfig; }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 178 "lexer.l"
{ lr_dir= df_quit; return tokv_word_quit; }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 179 "lexer.l"
{ lr_dir= df_eof; return tokv_word_eof; }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 180 "lexer.l"
{ lr_dir= df_if; return tokv_word_if; }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 181 "lexer.l"
{ lr_dir= df_catchquit; return tokv_word_catchquit; }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 182 "lexer.l"
{ lr_dir= df_errorspush; return tokv_word_errorspush; }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 183 "lexer.l"
{ lr_controlend= tokv_word_if; return tokv_word_elif; }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 184 "lexer.l"
{ lr_controlend= tokv_word_if; return tokv_word_else; }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 185 "lexer.l"
{ lr_controlend= tokv_word_if; return tokv_word_fi; }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 186 "lexer.l"
{ lr_controlend= tokv_word_catchquit; return tokv_word_hctac; }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 187 "lexer.l"
{ lr_controlend= tokv_word_errorspush; return tokv_word_srorre; }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 188 "lexer.l"
{ lr_parmcond= pcf_glob; return tokv_word_glob; }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 189 "lexer.l"
{ lr_parmcond= pcf_range; return tokv_word_range; }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 190 "lexer.l"
{ lr_parmcond= pcf_grep; return tokv_word_grep; }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 191 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_environment; return tokv_word_environment; }
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 192 "lexer.l"
{ lr_bispa= bispa_parameter; lr_bisexec= bisexec_parameter; return tokv_word_parameter; }
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 193 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_version; return tokv_word_version; }
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 194 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_toplevel; return tokv_word_toplevel; }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 195 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_override; return tokv_word_override; }
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 196 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_shutdown; return tokv_word_shutdown; }
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 197 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_reset; lr_dir= df_reset; return tokv_word_reset; }
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 198 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_execute; lr_dir= df_execute; return tokv_word_execute; }
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 199 "lexer.l"
{ lr_bispa= bispa_none; lr_bisexec= bisexec_help; return tokv_word_help; }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 200 "lexer.l"
{ lr_parameter= pf_service; return tokv_word_service; }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 201 "lexer.l"
{ lr_parameter= pf_callinguser; return tokv_word_callinguser; }
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 202 "lexer.l"
{ lr_parameter= pf_callinggroup; return tokv_word_callinggroup; }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 203 "lexer.l"
{ lr_parameter= pf_callingusershell; return tokv_word_callingusershell; }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 204 "lexer.l"
{ lr_parameter= pf_serviceuser; return tokv_word_serviceuser; }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 205 "lexer.l"
{ lr_parameter= pf_servicegroup; return tokv_word_servicegroup; }
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 206 "lexer.l"
{ lr_parameter= pf_serviceusershell; return tokv_word_serviceusershell; }
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 207 "lexer.l"
{ lr_loglevel= LOG_DEBUG; return tokv_syslog_debug; }
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 208 "lexer.l"
{ lr_loglevel= LOG_INFO; return tokv_syslog_info; }
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 209 "lexer.l"
{ lr_loglevel= LOG_NOTICE; return tokv_syslog_notice; }
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 210 "lexer.l"
{ lr_loglevel= LOG_WARNING; return tokv_syslog_warning; }
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 211 "lexer.l"
{ lr_loglevel= LOG_ERR; return tokv_syslog_err; }
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 212 "lexer.l"
{ lr_loglevel= LOG_CRIT; return tokv_syslog_crit; }
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 213 "lexer.l"
{ lr_loglevel= LOG_ALERT; return tokv_syslog_alert; }
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 214 "lexer.l"
{ lr_loglevel= LOG_EMERG; return tokv_syslog_emerg; }
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 215 "lexer.l"
{ lr_logfacility= LOG_AUTHPRIV; return tokv_syslog_authpriv; }
	YY_BREAK
case 70:
YY_RULE_SETUP
#line 216 "lexer.l"
{ lr_logfacility= LOG_CRON; return tokv_syslog_cron; }
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 217 "lexer.l"
{ lr_logfacility= LOG_DAEMON; return tokv_syslog_daemon; }
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 218 "lexer.l"
{ lr_logfacility= LOG_KERN; return tokv_syslog_kern; }
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 219 "lexer.l"
{ lr_logfacility= LOG_LPR; return tokv_syslog_lpr; }
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 220 "lexer.l"
{ lr_logfacility= LOG_MAIL; return tokv_syslog_mail; }
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 221 "lexer.l"
{ lr_logfacility= LOG_NEWS; return tokv_syslog_news; }
	YY_BREAK
case 76:
YY_RULE_SETUP
#line 222 "lexer.l"
{ lr_logfacility= LOG_SYSLOG; return tokv_syslog_syslog; }
	YY_BREAK
case 77:
YY_RULE_SETUP
#line 223 "lexer.l"
{ lr_logfacility= LOG_USER; return tokv_syslog_user; }
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 224 "lexer.l"
{ lr_logfacility= LOG_UUCP; return tokv_syslog_uucp; }
	YY_BREAK
case 79:
YY_RULE_SETUP
#line 225 "lexer.l"
{ lr_logfacility= LOG_LOCAL0; return tokv_syslog_local0; }
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 226 "lexer.l"
{ lr_logfacility= LOG_LOCAL1; return tokv_syslog_local1; }
	YY_BREAK
case 81:
YY_RULE_SETUP
#line 227 "lexer.l"
{ lr_logfacility= LOG_LOCAL2; return tokv_syslog_local2; }
	YY_BREAK
case 82:
YY_RULE_SETUP
#line 228 "lexer.l"
{ lr_logfacility= LOG_LOCAL3; return tokv_syslog_local3; }
	YY_BREAK
case 83:
YY_RULE_SETUP
#line 229 "lexer.l"
{ lr_logfacility= LOG_LOCAL4; return tokv_syslog_local4; }
	YY_BREAK
case 84:
YY_RULE_SETUP
#line 230 "lexer.l"
{ lr_logfacility= LOG_LOCAL5; return tokv_syslog_local5; }
	YY_BREAK
case 85:
YY_RULE_SETUP
#line 231 "lexer.l"
{ lr_logfacility= LOG_LOCAL6; return tokv_syslog_local6; }
	YY_BREAK
case 86:
YY_RULE_SETUP
#line 232 "lexer.l"
{ lr_logfacility= LOG_LOCAL7; return tokv_syslog_local7; }
	YY_BREAK
case 87:
YY_RULE_SETUP
#line 233 "lexer.l"
{ return tokv_word_read; }
	YY_BREAK
case 88:
YY_RULE_SETUP
#line 234 "lexer.l"
{ return tokv_word_write; }
	YY_BREAK
case 89:
YY_RULE_SETUP
#line 235 "lexer.l"
{ return tokv_dollar; }
	YY_BREAK
case 90:
YY_RULE_SETUP
#line 236 "lexer.l"
{  lr_max= lr_min= 0; return tokv_word_stdin; }
	YY_BREAK
case 91:
YY_RULE_SETUP
#line 237 "lexer.l"
{  lr_max= lr_min= 1; return tokv_word_stdout; }
	YY_BREAK
case 92:
YY_RULE_SETUP
#line 238 "lexer.l"
{  lr_max= lr_min= 2; return tokv_word_stderr; }
	YY_BREAK
case 93:
YY_RULE_SETUP
#line 239 "lexer.l"
{ return tokv_openparen; }
	YY_BREAK
case 94:
YY_RULE_SETUP
#line 240 "lexer.l"
{ return tokv_closeparen; }
	YY_BREAK
case 95:
YY_RULE_SETUP
#line 241 "lexer.l"
{ return tokv_not; }
	YY_BREAK
case 96:
YY_RULE_SETUP
#line 242 "lexer.l"
{ return tokv_and; }
	YY_BREAK
case 97:
YY_RULE_SETUP
#line 243 "lexer.l"
{ return tokv_or; }
	YY_BREAK
case 98:
YY_RULE_SETUP
#line 244 "lexer.l"
{ lr_dir= df_error; lr_loglevel= LOG_ERR; return tokv_word_error; }
	YY_BREAK
case 99:
YY_RULE_SETUP
#line 248 "lexer.l"
{
			  char *ep;
			  lr_min=lr_max= (int)strtoul(yytext,&ep,10);
			  assert(!*ep);
			  return tokv_ordinal;
			}
	YY_BREAK
case 100:
YY_RULE_SETUP
#line 254 "lexer.l"
{
			  char *ep;
			  lr_min= (int)strtoul(yytext,&ep,10);
			  assert(*ep == HYPHEN);
			  ep++;  assert(*ep);
			  lr_max= (int)strtoul(ep,&ep,10);
			  assert(!*ep);
			  if (lr_max < lr_min)
			    return parseerrprint("fd range has min > max");
			  return tokv_fdrange;
			}
	YY_BREAK
case 101:
YY_RULE_SETUP
#line 265 "lexer.l"
{
			  char *ep;
			  lr_min= (int)strtoul(yytext,&ep,10);
			  assert(*ep == HYPHEN);
			  ep++;  assert(!*ep);
			  lr_max=-1;
			  return tokv_fdstoend;
			}
	YY_BREAK
case 102:
/* rule 102 can match eol */
YY_RULE_SETUP
#line 273 "lexer.l"
countnewlines(); return tokv_lwsp;
	YY_BREAK
case 103:
YY_RULE_SETUP
#line 274 "lexer.l"
return tokv_lwsp;
	YY_BREAK
case 104:
/* rule 104 can match eol */
YY_RULE_SETUP
#line 275 "lexer.l"
cstate->lineno++; return tokv_newline;
	YY_BREAK
case 105:
/* rule 105 can match eol */
YY_RULE_SETUP
#line 276 "lexer.l"
cstate->lineno++; return tokv_newline;
	YY_BREAK
case 106:
YY_RULE_SETUP
#line 277 "lexer.l"
return parseerrprint("missing newline at eof after comment");
	YY_BREAK
case 107:
/* rule 107 can match eol */
YY_RULE_SETUP
#line 278 "lexer.l"
{
			  countnewlines();
			  return dequote(yytext);
			}
	YY_BREAK
case 108:
YY_RULE_SETUP
#line 282 "lexer.l"
return tokv_barestring;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
#line 283 "lexer.l"
return tokv_eof;
	YY_BREAK
case 109:
YY_RULE_SETUP
#line 284 "lexer.l"
return parseerrprint("misquoted or unterminated string");
	YY_BREAK
case 110:
YY_RULE_SETUP
#line 285 "lexer.l"
return parseerrprint("unexpected backslash");
	YY_BREAK
case 111:
YY_RULE_SETUP
#line 286 "lexer.l"
abort(); /* expect lex warning "rule cannot be matched" */
	YY_BREAK
case 112:
YY_RULE_SETUP
#line 289 "lexer.l"
ECHO;
	YY_BREAK
#line 1835 "<stdout>"

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( yywrap(  ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = (yytext_ptr);
	int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr) - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc( (void *) b->yy_ch_buf,
							 (yy_size_t) (b->yy_buf_size + 2)  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = NULL;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if (((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc(
			(void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, (yy_size_t) new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		YY_CURRENT_BUFFER_LVALUE->yy_buf_size = (int) (new_size - 2);
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	yy_state_type yy_current_state;
	char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 587 )
				yy_c = yy_meta[yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	int yy_is_jam;
    	char *yy_cp = (yy_c_buf_p);

	YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 587 )
			yy_c = yy_meta[yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
	yy_is_jam = (yy_current_state == 586);

		return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_UNPUT

#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (int) ((yy_c_buf_p) - (yytext_ptr));
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap(  ) )
						return 0;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer( yyin, YY_BUF_SIZE );
	}

	yy_init_buffer( YY_CURRENT_BUFFER, input_file );
	yy_load_buffer_state(  );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state(  );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc( (yy_size_t) (b->yy_buf_size + 2)  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * 
 */
    void yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree( (void *) b->yy_ch_buf  );

	yyfree( (void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state(  );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack();

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state(  );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state(  );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack (void)
{
	yy_size_t num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		(yy_buffer_stack) = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));

		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (const char * yystr )
{
    
	return yy_scan_bytes( yystr, (int) strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (const char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) (_yybytes_len + 2);
	buf = (char *) yyalloc( n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yynoreturn yy_fatal_error (const char* msg )
{
			fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int yyget_lineno  (void)
{
    
    return yylineno;
}

/** Get the input stream.
 * 
 */
FILE *yyget_in  (void)
{
        return yyin;
}

/** Get the output stream.
 * 
 */
FILE *yyget_out  (void)
{
        return yyout;
}

/** Get the length of the current token.
 * 
 */
int yyget_leng  (void)
{
        return yyleng;
}

/** Get the current token.
 * 
 */

char *yyget_text  (void)
{
        return yytext;
}

/** Set the current line number.
 * @param _line_number line number
 * 
 */
void yyset_lineno (int  _line_number )
{
    
    yylineno = _line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * 
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  _in_str )
{
        yyin = _in_str ;
}

void yyset_out (FILE *  _out_str )
{
        yyout = _out_str ;
}

int yyget_debug  (void)
{
        return yy_flex_debug;
}

void yyset_debug (int  _bdebug )
{
        yy_flex_debug = _bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = NULL;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = NULL;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer( YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, const char * s2, int n )
{
		
	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (const char * s )
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size )
{
			return malloc(size);
}

void *yyrealloc  (void * ptr, yy_size_t  size )
{
		
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void yyfree (void * ptr )
{
			free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 289 "lexer.l"


const char *const builtinservicehelpstrings[]= {
  "environment",
  "parameter <parameter>",
  "version",
  "toplevel",
  "override",
  "shutdown",
  "reset",
  "execute",
  "help",
   0
};

#include "parser.c"


